From: Imre Deak <imre.deak@intel.com>
To: intel-gfx@lists.freedesktop.org, intel-xe@lists.freedesktop.org
Subject: [PATCH v3 5/5] drm/i915/crt: Use intel_hpd_block/unblock() instead of intel_hpd_disable/enable()
Date: Wed, 26 Feb 2025 19:19:23 +0200 [thread overview]
Message-ID: <20250226171924.2646997-6-imre.deak@intel.com> (raw)
In-Reply-To: <20250226171924.2646997-1-imre.deak@intel.com>
intel_hpd_disable/enable() have the same purpose as
intel_hpd_block/unblock(), except that disable/enable will drop any HPD
IRQs which were triggered while the HPD was disabled, while
block/unblock will handle such IRQs after the IRQ handling is unblocked.
Use intel_hpd_block/unblock() for crt as well, by adding a helper to
explicitly clear any pending IRQs before unblocking.
v2:
- Handle encoders without a port assigned to them.
- Rebase on change in intel_hpd_suspend() documentation.
v3:
- Rebase on the suspend/resume -> block/unblock rename change.
- Clear the pending events only after all encoders have unblocked the
HPD handling.
- Clear the short/long port events for all encoders using the given HPD
pin.
Signed-off-by: Imre Deak <imre.deak@intel.com>
---
drivers/gpu/drm/i915/display/intel_crt.c | 7 +--
drivers/gpu/drm/i915/display/intel_hotplug.c | 62 +++++++++++---------
drivers/gpu/drm/i915/display/intel_hotplug.h | 3 +-
3 files changed, 37 insertions(+), 35 deletions(-)
diff --git a/drivers/gpu/drm/i915/display/intel_crt.c b/drivers/gpu/drm/i915/display/intel_crt.c
index 321580b095e7d..e8a2f28e662ec 100644
--- a/drivers/gpu/drm/i915/display/intel_crt.c
+++ b/drivers/gpu/drm/i915/display/intel_crt.c
@@ -531,8 +531,6 @@ static bool valleyview_crt_detect_hotplug(struct drm_connector *connector)
{
struct intel_display *display = to_intel_display(connector->dev);
struct intel_crt *crt = intel_attached_crt(to_intel_connector(connector));
- struct drm_i915_private *dev_priv = to_i915(connector->dev);
- bool reenable_hpd;
u32 adpa;
bool ret;
u32 save_adpa;
@@ -549,7 +547,7 @@ static bool valleyview_crt_detect_hotplug(struct drm_connector *connector)
*
* Just disable HPD interrupts here to prevent this
*/
- reenable_hpd = intel_hpd_disable(dev_priv, crt->base.hpd_pin);
+ intel_hpd_block(&crt->base);
save_adpa = adpa = intel_de_read(display, crt->adpa_reg);
drm_dbg_kms(display->drm,
@@ -576,8 +574,7 @@ static bool valleyview_crt_detect_hotplug(struct drm_connector *connector)
drm_dbg_kms(display->drm,
"valleyview hotplug adpa=0x%x, result %d\n", adpa, ret);
- if (reenable_hpd)
- intel_hpd_enable(dev_priv, crt->base.hpd_pin);
+ intel_hpd_clear_and_unblock(&crt->base);
return ret;
}
diff --git a/drivers/gpu/drm/i915/display/intel_hotplug.h b/drivers/gpu/drm/i915/display/intel_hotplug.h
index 5f9857136f5e3..f189b871904ed 100644
--- a/drivers/gpu/drm/i915/display/intel_hotplug.h
+++ b/drivers/gpu/drm/i915/display/intel_hotplug.h
@@ -26,10 +26,9 @@ void intel_hpd_init(struct drm_i915_private *dev_priv);
void intel_hpd_init_early(struct drm_i915_private *i915);
void intel_hpd_cancel_work(struct drm_i915_private *dev_priv);
enum hpd_pin intel_hpd_pin_default(enum port port);
-bool intel_hpd_disable(struct drm_i915_private *dev_priv, enum hpd_pin pin);
-void intel_hpd_enable(struct drm_i915_private *dev_priv, enum hpd_pin pin);
void intel_hpd_block(struct intel_encoder *encoder);
void intel_hpd_unblock(struct intel_encoder *encoder);
+void intel_hpd_clear_and_unblock(struct intel_encoder *encoder);
void intel_hpd_debugfs_register(struct drm_i915_private *i915);
void intel_hpd_enable_detection_work(struct drm_i915_private *i915);
diff --git a/drivers/gpu/drm/i915/display/intel_hotplug.c b/drivers/gpu/drm/i915/display/intel_hotplug.c
index 6c6b7d661d639..b0079c6efa83d 100644
--- a/drivers/gpu/drm/i915/display/intel_hotplug.c
+++ b/drivers/gpu/drm/i915/display/intel_hotplug.c
@@ -1017,33 +1017,6 @@ void intel_hpd_cancel_work(struct drm_i915_private *dev_priv)
drm_dbg_kms(&dev_priv->drm, "Hotplug detection work still active\n");
}
-bool intel_hpd_disable(struct drm_i915_private *dev_priv, enum hpd_pin pin)
-{
- bool ret = false;
-
- if (pin == HPD_NONE)
- return false;
-
- spin_lock_irq(&dev_priv->irq_lock);
- if (dev_priv->display.hotplug.stats[pin].state == HPD_ENABLED) {
- dev_priv->display.hotplug.stats[pin].state = HPD_DISABLED;
- ret = true;
- }
- spin_unlock_irq(&dev_priv->irq_lock);
-
- return ret;
-}
-
-void intel_hpd_enable(struct drm_i915_private *dev_priv, enum hpd_pin pin)
-{
- if (pin == HPD_NONE)
- return;
-
- spin_lock_irq(&dev_priv->irq_lock);
- dev_priv->display.hotplug.stats[pin].state = HPD_ENABLED;
- spin_unlock_irq(&dev_priv->irq_lock);
-}
-
static void queue_work_for_missed_irqs(struct drm_i915_private *i915)
{
struct intel_display *display = to_intel_display(&i915->drm);
@@ -1121,7 +1094,8 @@ static bool unblock_hpd_pin(struct intel_display *display, enum hpd_pin pin)
* drm_connector_funcs::detect()) remains allowed, for instance as part of
* userspace connector probing, or DRM core's connector polling.
*
* The call must be followed by calling intel_hpd_unblock().
*
* Note that the handling of HPD IRQs for another encoder using the same HPD
* pin as that of @encoder will be also blocked.
@@ -1172,6 +1146,38 @@ void intel_hpd_unblock(struct intel_encoder *encoder)
spin_unlock_irq(&i915->irq_lock);
}
+/**
+ * intel_hpd_clear_and_unblock - Unblock handling of new HPD IRQs on an HPD pin
+ * @encoder: Encoder to unblock the HPD handling for
+ *
+ * Unblock the handling of HPD IRQs on the HPD pin of @encoder, which was
+ * previously blocked by intel_hpd_block(). Any HPD IRQ raised on the
+ * HPD pin while it was blocked will be cleared, handling only new IRQs.
+ */
+void intel_hpd_clear_and_unblock(struct intel_encoder *encoder)
+{
+ struct intel_display *display = to_intel_display(encoder);
+ struct drm_i915_private *i915 = to_i915(display->drm);
+ struct intel_hotplug *hotplug = &display->hotplug;
+ u32 blocked_ports;
+
+ if (encoder->hpd_pin == HPD_NONE)
+ return;
+
+ blocked_ports = hpd_pin_mask_to_ports(display, BIT(encoder->hpd_pin));
+
+ spin_lock_irq(&i915->irq_lock);
+
+ if (unblock_hpd_pin(display, encoder->hpd_pin)) {
+ hotplug->event_bits &= ~BIT(encoder->hpd_pin);
+ hotplug->retry_bits &= ~BIT(encoder->hpd_pin);
+ hotplug->short_port_mask &= ~blocked_ports;
+ hotplug->long_port_mask &= ~blocked_ports;
+ }
+
+ spin_unlock_irq(&i915->irq_lock);
+}
+
void intel_hpd_enable_detection_work(struct drm_i915_private *i915)
{
spin_lock_irq(&i915->irq_lock);
@@ -1121,7 +1094,8 @@ static bool unblock_hpd_pin(struct intel_display *display, enum hpd_pin pin)
* userspace connector probing, or DRM core's connector polling.
*
- * The call must be followed by calling intel_hpd_unblock().
+ * The call must be followed by calling intel_hpd_unblock(), or
+ * intel_hpd_clear_and_unblock().
*
* Note that the handling of HPD IRQs for another encoder using the same HPD
--
2.44.2
next prev parent reply other threads:[~2025-02-26 17:18 UTC|newest]
Thread overview: 17+ messages / expand[flat|nested] mbox.gz Atom feed top
2025-02-26 17:19 [PATCH v3 0/5] drm/dp: Fix link training interrupted by HPD pulse Imre Deak
2025-02-26 17:19 ` [PATCH v3 1/5] drm/i915/hpd: Let an HPD pin be in the disabled state when handling missed IRQs Imre Deak
2025-02-26 17:19 ` [PATCH v3 2/5] drm/i915/hpd: Add support for blocking the IRQ handling on an HPD pin Imre Deak
2025-02-26 17:35 ` [PATCH v4 " Imre Deak
2025-02-28 17:25 ` Ville Syrjälä
2025-02-28 20:04 ` Imre Deak
2025-02-26 17:19 ` [PATCH v3 3/5] drm/i915/dp: Fix link training interrupted by a short HPD pulse Imre Deak
2025-02-26 17:19 ` [PATCH v3 4/5] drm/i915/dp: Queue a link check after link training is complete Imre Deak
2025-02-26 17:19 ` Imre Deak [this message]
2025-02-26 19:23 ` ✓ CI.Patch_applied: success for drm/dp: Fix link training interrupted by HPD pulse (rev5) Patchwork
2025-02-26 19:23 ` ✓ CI.checkpatch: " Patchwork
2025-02-26 19:24 ` ✓ CI.KUnit: " Patchwork
2025-02-26 19:41 ` ✓ CI.Build: " Patchwork
2025-02-26 19:43 ` ✓ CI.Hooks: " Patchwork
2025-02-26 19:45 ` ✗ CI.checksparse: warning " Patchwork
2025-02-26 20:04 ` ✓ Xe.CI.BAT: success " Patchwork
2025-02-26 21:17 ` ✗ Xe.CI.Full: failure " Patchwork
Reply instructions:
You may reply publicly to this message via plain-text email
using any one of the following methods:
* Save the following mbox file, import it into your mail client,
and reply-to-all from there: mbox
Avoid top-posting and favor interleaved quoting:
https://en.wikipedia.org/wiki/Posting_style#Interleaved_style
* Reply using the --to, --cc, and --in-reply-to
switches of git-send-email(1):
git send-email \
--in-reply-to=20250226171924.2646997-6-imre.deak@intel.com \
--to=imre.deak@intel.com \
--cc=intel-gfx@lists.freedesktop.org \
--cc=intel-xe@lists.freedesktop.org \
/path/to/YOUR_REPLY
https://kernel.org/pub/software/scm/git/docs/git-send-email.html
* If your mail client supports setting the In-Reply-To header
via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line
before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox